import { create } from "./create.js";
import { createFrame } from "./frame.js";
import { getframe, suspend } from "../instructions.js";
/**
 * Get the scope of the currently running {@link Operation}.
 *
 * @returns an operation that yields the current scope
 */
export function* useScope() {
    let frame = yield* getframe();
    let [scope] = createScope(frame);
    return scope;
}
export function createScope(frame) {
    let parent = frame ?? createFrame({ operation: suspend });
    let scope = create("Scope", {}, {
        run(operation) {
            let frame = parent.createChild(operation);
            frame.enter();
            return frame.getTask();
        },
        get(context) {
            let { key, defaultValue } = context;
            return (parent.context[key] ?? defaultValue);
        },
        set(context, value) {
            let { key } = context;
            parent.context[key] = value;
            return value;
        },
    });
    parent.enter();
    return [scope, parent.getTask().halt];
}
//# sourceMappingURL=scope.js.map